library(pROC)
library(doSNOW)
library(parallel)
library(matrixStats)
set.seed(2018)
load('PretermData.rda')

FilterFeatures <- function(X)
{
  library(matrixStats)
  uCsite=unique(Csite)
  sds=colSds(X)
  index=which(sds==0)
  for(s in seq(length(uCsite))){
    siteindex=which(Csite==uCsite[s])
    sds=colSds(X[siteindex,])
    index=c(index, which(sds==0))
  }
  return(unique(index))
}
CTranscriptomics = CTranscriptomics[,-FilterFeatures(CTranscriptomics)]
Metabolomics = CMetabolomics[,which(colSds(CMetabolomics)!=0)]
colnames(Metabolomics)=NULL
CProteomics = CProteomics[,which(colSds(CProteomics)!=0)]

cl <- makeSOCKcluster(detectCores())
registerDoSNOW(cl)

xxx<-function(X, Y, site, Ccohort, i)
{
  library(randomForest)
  set.seed(2018+123*i)
  XX=X[-i,]
  YY=Y[-i]
  CC=Ccohort[-i]
  XT=rbind(X[i,], X[i,])
  iii=which(site[-i]==site[i])
  pvs2=sapply(1:ncol(X), function(j) ( wilcox.test( XX[iii,j] ~ as.factor(CC[iii]))$p.value ) )
  cutoff=0.01
  ColFeatInd2=which(pvs2<cutoff)
  while(length(ColFeatInd2)<2)
  {
    ColFeatInd2=which(pvs2<cutoff)
    cutoff=cutoff+0.01
  }
  RF2=randomForest(XX[, ColFeatInd2], as.factor(CC))
  ret=list()
  ret$p2 = predict(RF2, XT[, ColFeatInd2], type='prob')[1]
  ret$ptrain2 = predict(RF2, XX[, ColFeatInd2], type='prob')
  ret$coef2=rep(0, ncol(XX))
  ret$coef2[ColFeatInd2]=importance(RF2)
  return(ret)
}
twoLayer<-function(X, Y, site, Ccohort, i)
{
  ret=list()
  ret$Layer1=xxx(X, Y, site, Ccohort, i)
  prdV=list()
  for(j in seq(nrow(X[-i,])))
    prdV[[j]]=xxx(X[-i,], Y[-i], site[-i], Ccohort[-i], j)
  ret$Layer2=prdV
  return(ret)
}
Yh=CGAdelivery
prdP=foreach(i=seq(nrow(CProteomics))) %dopar% twoLayer(CProteomics, Yh, Csite, Ccohort, i)
prdM=foreach(i=seq(nrow(Metabolomics))) %dopar% twoLayer(Metabolomics, Yh, Csite, Ccohort, i)
prdT=foreach(i=seq(nrow(CTranscriptomics))) %dopar% twoLayer(CTranscriptomics, Yh, Csite, Ccohort, i)

myPvs=vector()
myPvs1=vector()
myPvs2=vector()
ppp=vector()
for(i in seq(nrow(CProteomics)))
  ppp[i] = prdP[[i]]$Layer1$p2
myPvs[3]=roc(Ccohort, ppp)$auc
myPvs1[3]=-log10(cor.test(Yh, ppp, method = 'spearman')$p.value)
myPvs2[3]=-log10(wilcox.test(ppp ~ as.factor(Ccohort))$p.value)

mmm=vector()
for(i in seq(nrow(Metabolomics)))
  mmm[i] = prdM[[i]]$Layer1$p2
myPvs[2]=roc(Ccohort, mmm)$auc
myPvs1[2]=-log10(cor.test(Yh, mmm, method = 'spearman')$p.value)
myPvs2[2]=-log10(wilcox.test(mmm ~ as.factor(Ccohort))$p.value)

ttt=vector()
for(i in seq(nrow(CTranscriptomics)))
  ttt[i] = prdT[[i]]$Layer1$p2
myPvs[1]=roc(Ccohort, ttt)$auc
myPvs1[1]=-log10(cor.test(Yh, ttt, method = 'spearman')$p.value)
myPvs2[1]=-log10(wilcox.test(ttt ~ as.factor(Ccohort))$p.value)

CMBW=vector()
for(i in seq(nrow(CTranscriptomics)) )
{
  prf=vector()
  iii=which(Csite[-i]==Csite[i])
  pppT=vector()
  mmmT=vector()
  tttT=vector()
  for(j in seq(80))
  {
    pppT[j]=prdP[[i]]$Layer2[[j]]$p2
    mmmT[j]=prdM[[i]]$Layer2[[j]]$p2
    tttT[j]=prdT[[i]]$Layer2[[j]]$p2
  }
  prf[1]=wilcox.test(tttT[iii] ~ as.factor(Ccohort[-i][iii]))$p.value
  prf[2]=wilcox.test(mmmT[iii] ~ as.factor(Ccohort[-i][iii]))$p.value
  prf[3]=wilcox.test(pppT[iii] ~ as.factor(Ccohort[-i][iii]))$p.value
  prf=exp(-log10(prf))
  CMBW[i]=(prf[1]*ttt[i]+prf[2]*mmm[i]+prf[3]*ppp[i])/sum(prf)
}

myPvs[4]=roc(Ccohort, CMBW)$auc
myPvs1[4]=-log10(cor.test(Yh, CMBW, method = 'spearman')$p.value)
myPvs2[4]=-log10(wilcox.test(CMBW ~ as.factor(Ccohort))$p.value)
mydatasets=c('Cellfree RNA','PlasmaLuminex','SerumLuminex','Microbiome','ImmuneSystem','Metabolomics', 'PlasmaSomalogic', 'Merging', 'WeightedAverage')
mycols=c('#FFD600','#F44336','#43A047','#1E88E5','#AB47BC', '#3F51B5','#F57C00', 'gray', 'black')
myInd=c(1, 6, 7, 9)

par(mfrow = c(2, 2))
par(mar=c(10,5,5,0))
barplot(myPvs, las=2, ylab='Area Under Curve', col= mycols[myInd], names.arg = mydatasets[myInd], ylim=c(.50, .85), xpd = FALSE)
par(mar=c(10,5,5,0))
barplot(myPvs2, las=2, ylab='Wilcox classification -log10(p-value)', col= mycols[myInd], names.arg = mydatasets[myInd])
abline(h=-log10(0.05), col="red", lty=2, lwd=3)
par(mar=c(10,5,5,0))
barplot(myPvs1, las=2, ylab='Spearman regression -log10(p-value)', col= mycols[myInd], names.arg = mydatasets[myInd])
abline(h=-log10(0.05), col="red", lty=2, lwd=3)
